package com.miya.draw.manager;

import cn.hutool.core.bean.copier.BeanCopier;
import cn.hutool.core.bean.copier.CopyOptions;
import cn.hutool.core.date.LocalDateTimeUtil;
import com.google.common.collect.Lists;
import com.miya.draw.config.constant.ConStants;
import com.miya.draw.config.exception.BusinessException;
import com.miya.draw.entity.*;
import com.miya.draw.enums.*;
import com.miya.draw.model.DrawModel;
import com.miya.draw.request.DrawUserInfoGainRequest;
import com.miya.draw.request.MinePrizeGainRequest;
import com.miya.draw.request.PrizeDrawListRequest;
import com.miya.draw.service.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 抽奖Manager
 *
 * @author jack
 */
@Component
@Slf4j
public class DrawManagerImpl implements IDrawManager {

    @Autowired
    private IAccountService accountService;


    @Autowired
    private IAccountPrizeService accountPrizeService;

    @Autowired
    private ISyncTradeService syncTradeService;

    @Autowired
    private IActivityStoreService activityStoreService;

    @Autowired
    private IActivityGoodsService activityGoodsService;

    @Autowired
    private RedissonClient redissonClient;

    @Autowired
    private IPrizeService prizeService;

    @Autowired

    private IOtherManager otherManager;

    @Autowired
    private IActivityService activityService;

    @Override
    public DrawModel gainActivityUserInfo(DrawUserInfoGainRequest request) {
        try {
            Account account = new Account();
            account.setName(StringUtils.isNotBlank(request.getUserName()) ? request.getUserName() : request.getPhone());
            account.setPhone(request.getPhone());
            account.setToken(request.getToken());
            accountService.saveOrUpdateNotById(account);
        } catch (Exception e) {
            log.error("保存用户信息出错:{}", e.getMessage());
        }
        Activity activity = activityService.getById(request.getActivityId());
        if(activity == null){
            throw  new BusinessException(ErrorEnum.SYSTEM_EXP, "未获取活动有效配置");
        }
        LocalDateTime nowTime = LocalDateTime.now();
        LocalDateTime start = nowTime.minusDays(activity.getValidTime()-1);
        start = LocalDateTimeUtil.beginOfDay(start);
        LocalDateTime end = LocalDateTimeUtil.endOfDay(nowTime);
        Integer maxDrawCountLimit = activity.getDrawMaxCount();
        Long mchId = activity.getMchId();
        //获取已抽奖信息
        Integer alreadyDrawCount = accountPrizeService.gainAlreadyDrawCount(request.getPhone(), activity.getId(), start, end);
        //获取订单信息
        List<SyncTrade> buyGoodList = getTradeInfo(request.getPhone(), start, end,activity);
        BigDecimal buyTotal = gainBuyTotal(buyGoodList);
        Long purchasedAmount = buyTotal.multiply(BigDecimal.valueOf(100)).longValue();
        Integer drawCountTotal =0;
        if(activity.getRaffle()== RaffleEnum.AMOUNTUNIT.getCode()){
             buyTotal = getAmountTotal(buyGoodList);
             drawCountTotal = buyTotal.divide(new BigDecimal(activity.getAmountUnit()), RoundingMode.DOWN).intValue();
            //足量抽奖
        }else{
            drawCountTotal = buyTotal.divide(new BigDecimal(activity.getCountUnit()), RoundingMode.DOWN).intValue();

        }
        drawCountTotal = Math.min(activity.getDrawMaxCount(), drawCountTotal);
        drawCountTotal = drawCountTotal - alreadyDrawCount;
        Integer joinActivityPeopleNum = accountPrizeService.countJoinActivityPeople(activity.getId());
        return DrawModel.builder()
                .alreadyDrawCount(alreadyDrawCount)
                .drawCountTotal(drawCountTotal>=0?drawCountTotal:0)
                .maxDrawCountLimit(maxDrawCountLimit)
                .numOfDrawCount(Math.max(0, maxDrawCountLimit - alreadyDrawCount))
                .purchasedAmount(purchasedAmount)
                .taxAmountCache(Math.max(0L, BigDecimal.valueOf(activity.getMaxMoney()).subtract(buyTotal).multiply(BigDecimal.valueOf(100)).longValue()))
                .numOfJoinActivityPeople(joinActivityPeopleNum)
                .build();
    }

    /**
     * 获取购买的交易总额
     *
     * @param buyGoodList 购买商品列表
     * @return 购买总金额
     */
    private BigDecimal gainBuyTotal(List<SyncTrade> buyGoodList) {
        BigDecimal buyTotal = buyGoodList.stream().filter(x -> TradeTypeEnum.SALE.getType().equalsIgnoreCase(x.getType())).map(syncTrade -> syncTrade.getStdAmount().subtract(syncTrade.getFavAmount()))
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        BigDecimal refundTotal = buyGoodList.stream().filter(x -> TradeTypeEnum.REFUND.getType().equalsIgnoreCase(x.getType())).map(syncTrade -> syncTrade.getStdAmount().subtract(syncTrade.getFavAmount()))
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        buyTotal = buyTotal.compareTo(refundTotal) >= 0 ? buyTotal.subtract(refundTotal) : BigDecimal.ZERO;
        return buyTotal;
    }

    /**
     * 获取购买交易总数
     * @param buyGoodList
     * @return
     */
    private BigDecimal getAmountTotal(List<SyncTrade> buyGoodList){
        BigDecimal buyTotal =  buyGoodList.stream().filter(x -> TradeTypeEnum.SALE.getType().equalsIgnoreCase(x.getType())).map(syncTrade->syncTrade.getQty())
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        BigDecimal refundTotal = buyGoodList.stream().filter(x -> TradeTypeEnum.REFUND.getType().equalsIgnoreCase(x.getType())).map(syncTrade->syncTrade.getQty())
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        buyTotal = buyTotal.compareTo(refundTotal) >= 0 ? buyTotal.subtract(refundTotal) : BigDecimal.ZERO;
        return buyTotal;
    }

    @Override
    public List<DrawModel> listMinePrizeList( MinePrizeGainRequest request) {
        Activity activity = activityService.getById(request.getActivityId());
        if(activity == null){
            throw  new BusinessException(ErrorEnum.SYSTEM_EXP, "未获取活动有效配置");
        }
        List<AccountPrize> prizeList = accountPrizeService.listMinePrize(request.getPhone(), request.getActivityId());
        return getDrawModels(prizeList);
    }

    @Override
    public void updateMinePrizeAddress(String phone, Long myPrizeId, String consigneeName, String consigneePhone, String consigneeAddress, String consigneeIdcard) {
        AccountPrize accountPrize = accountPrizeService.getById(myPrizeId);
        // 有奖品 手机号正确 且信息没填
        if (!Objects.isNull(accountPrize) && phone.equals(accountPrize.getAccountPhone()) && PrizeTypeEnum.PHYSICAL.getCode().equals(accountPrize.getPrizeType())) {
            if (StringUtils.isBlank(accountPrize.getConsigneeName()) && StringUtils.isBlank(accountPrize.getConsigneePhone())
                    && StringUtils.isBlank(accountPrize.getConsigneeAddress()) && StringUtils.isBlank(accountPrize.getConsigneeIdcard())) {
                AccountPrize prizeUpdate = new AccountPrize();
                prizeUpdate.setId(myPrizeId);
                prizeUpdate.setConsigneeAddress(consigneeAddress);
                prizeUpdate.setConsigneeIdcard(consigneeIdcard);
                prizeUpdate.setConsigneeName(consigneeName);
                prizeUpdate.setConsigneePhone(consigneePhone);
                // 填写信息后重新同步
                prizeUpdate.setSyncStatus(YesOrNoEnum.NO.getCode());
                accountPrizeService.saveOrUpdate(prizeUpdate);
            } else {
                throw new BusinessException(ErrorEnum.PARAM_CHECK_EXP, "收货信息已填写!");
            }
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public DrawModel drawClick(String phone,Long activityId ) throws InterruptedException {
        // 同一用户不可多次重复点击
        RLock lock = redissonClient.getLock(buildKey(phone));
        try {
            lock.tryLock(5, 10, TimeUnit.SECONDS);
            //1 判断活动是否有效
            Activity activity = activityService.getById(activityId);
            if(activity == null){
                throw new BusinessException(ErrorEnum.DRAW_CHECK_EXP, "活动不存在或已失效");
            }
            Date now = new Date();
            if(now.before(activity.getStartTime())){
                throw new BusinessException(ErrorEnum.DRAW_CHECK_EXP, "抱歉，抽奖活动还未开始！");
            }
            if(now.after(activity.getEndTime())){
                throw new BusinessException(ErrorEnum.DRAW_CHECK_EXP, "抱歉，抽奖活动已经结束！");
            }
            // 2 验证是否可以抽奖
            LocalDateTime nowTime = LocalDateTime.now();
            LocalDateTime start = nowTime.minusDays(activity.getValidTime()-1);
            start = LocalDateTimeUtil.beginOfDay(start);
            LocalDateTime end = LocalDateTimeUtil.endOfDay(nowTime);
            Long mchId = activity.getMchId();
            List<SyncTrade> buyGoodList = getTradeInfo(phone,start,end,activity);
             int drawCountTotal =0;
            //3 判断抽奖方式
            //足量抽奖
            if(activity.getRaffle()== RaffleEnum.AMOUNTUNIT.getCode()){
                BigDecimal buyTotal = getAmountTotal(buyGoodList);
                drawCountTotal = buyTotal.divide(new BigDecimal(activity.getAmountUnit()), RoundingMode.DOWN).intValue();
                drawCountTotal = Math.min(activity.getDrawMaxCount(), drawCountTotal);
                if (drawCountTotal <= 0) {
                    throw new BusinessException(ErrorEnum.DRAW_CHECK_EXP, "抱歉，您购买的活动商品不足，已无抽奖次数");
                }
                //满额抽奖
            }else{
                BigDecimal buyTotal = gainBuyTotal(buyGoodList);
                drawCountTotal = buyTotal.divide(new BigDecimal(activity.getCountUnit()), RoundingMode.DOWN).intValue();
                drawCountTotal = Math.min(activity.getDrawMaxCount(), drawCountTotal);
                if (drawCountTotal <= 0) {
                    throw new BusinessException(ErrorEnum.DRAW_CHECK_EXP, "抱歉，你的购物金额不足，已无抽奖次数");
                }
            }

            Integer alreadyDrawCount = accountPrizeService.gainAlreadyDrawCount(phone, activityId, start, end);
            // 可抽奖次数> 已抽奖次数
            Prize prize;
            AccountPrize accountPrize;
            if (drawCountTotal - alreadyDrawCount > 0) {
                // 获得奖品
                List<Prize> prizeList = gainPrizeInfo(phone,activityId);
                // 抽奖
                prize = selectCoupon(prizeList);
                // 乐观锁效验抽到的奖品它合不合规
                if (Integer.valueOf(1).equals(prizeService.updateCount(prize))) {
                    log.info("phone={}的用户抽中了={}", phone, prize);
                } else {
                    // 测试update=0的是否也会回滚
                    prize = prizeList.stream().filter(x -> PrizeTypeEnum.VIRTUAL.getCode().equals(x.getPrizeType())).max(Comparator.comparing(Prize::getId))
                            .orElseThrow(() -> new BusinessException(ErrorEnum.DRAW_CHECK_EXP, "当前抽奖人数过多，请稍候重试!"));
                }
                // 记录抽奖信息
                accountPrize = new AccountPrize();
                accountPrize.setActivityId(activityId);
                accountPrize.setPrizeId(prize.getId());
                accountPrize.setAccountName(phone);
                accountPrize.setAccountPhone(phone);
                accountPrize.setPrizeName(prize.getPrizeName());
                accountPrize.setPrizeType(prize.getPrizeType());
                accountPrizeService.saveOrUpdate(accountPrize);
                // 发券
                if (PrizeTypeEnum.VIRTUAL.getCode().equals(prize.getPrizeType()) && StringUtils.isNotBlank(prize.getCouponId())) {
                    otherManager.sendCoupon(prize.getCouponId(), phone, accountPrize.getId());
                }
            } else {
                if (alreadyDrawCount >= activity.getDrawMaxCount()) {
                    throw new BusinessException(ErrorEnum.DRAW_CHECK_EXP, "抱歉,你的今日可抽奖次数已用完,请明日再来哦");
                } else {
                    throw new BusinessException(ErrorEnum.DRAW_CHECK_EXP, "抱歉，你的购物金额或购买活动商品不足，已无抽奖次数");
                }
            }
            // 2 奖品
            BeanCopier<DrawModel> beanCopier = BeanCopier.create(prize, DrawModel.builder().build(), CopyOptions.create());
            DrawModel drawModel = beanCopier.copy();
            drawModel.setMyPrizeId(accountPrize.getId());
            drawModel.setId(prize.getId());
            return drawModel;
        } catch (Exception e) {
            log.error("抽奖异常:{}", e.getMessage());
            throw e;
        } finally {
            lock.unlock();
        }
    }

    public static void main(String[] args) {
        LocalDateTime nowTime = LocalDateTime.now();
        LocalDateTime start = nowTime.minusDays(1-1);
        start = LocalDateTimeUtil.beginOfDay(start);
        LocalDateTime end = LocalDateTimeUtil.endOfDay(nowTime);

    }

    /**
     * 获取参与活动用户的订单数据
     * @return
     */
    public List<SyncTrade> getTradeInfo(String phone, LocalDateTime start,LocalDateTime end,Activity activity ){
        Long mchId = activity.getMchId();
        Long activityId = activity.getId();
        /**
         * 活动商品
         */
        List<String> listAllActivityGoods = null;
        List<String> storeCodeList = null;
        //限制商品时获取限制或要剔除的商品
        if(activity.getActivityGoodsStatus() == ActivityStoreAndGoodsStatusEnum.CONFINE.getCode() ||
                activity.getActivityGoodsStatus() == ActivityStoreAndGoodsStatusEnum.ELIMINATE.getCode()){
            listAllActivityGoods = activityGoodsService.listAllActivityGoods(activityId);
        }

        if(activity.getActivityStoreStatus() == ActivityStoreAndGoodsStatusEnum.CONFINE.getCode() ||
                activity.getActivityStoreStatus() == ActivityStoreAndGoodsStatusEnum.ELIMINATE.getCode()){
            storeCodeList = activityStoreService.listAllActivityStoreId(activityId);
        }

        return  syncTradeService.listCanDrawGoods(mchId,phone, start, end, storeCodeList, listAllActivityGoods,activity);
    }

    /**
     * 用户限中奖品后重新获取奖品List
     * 1. 获取限制的奖品Id
     * 2. 查询当前号码是否满足中奖限制
     * 3. 如果满足中奖限制，将该奖品剔除
     * @param
     */
    public List<Prize>  gainPrizeInfo(String phone, Long activityId){
        List<Prize> prizeList = prizeService.listAllPrizes(activityId);
        if (CollectionUtils.isEmpty(prizeList)) {
            throw new BusinessException(ErrorEnum.DRAW_CHECK_EXP, "活动已结束!");
        }
        //获取限制抽奖的奖品
        List<Prize> restrictPrizeList = prizeList.stream().filter(p->p.getAstrict()!=0 ).collect(Collectors.toList());
        if(!CollectionUtils.isEmpty(restrictPrizeList)){
            List<Long> restrictPrizeId = restrictPrizeList.stream().map(Prize::getId).collect(Collectors.toList());
            List<AccountPrize> accountPrizeList = accountPrizeService.gainAlreadyDrawCountByPrize(phone,activityId,restrictPrizeId);
            //不可以抽奖的奖品Id
            List<Long> prizeIdList = Lists.newArrayList();
            if(!CollectionUtils.isEmpty(accountPrizeList)){
                for(AccountPrize account:accountPrizeList){
                    for (Prize prize:restrictPrizeList){
                        //判断是否已经达到抽奖限制次数
                        if(account.getPrizeId().equals(prize.getId()) && account.getCount()>= prize.getAstrict()){
                            prizeIdList.add(prize.getId());
                        }
                    }
                }
            }
            if(!CollectionUtils.isEmpty(prizeIdList)){
                //剔除已限制抽奖的奖品
                prizeList= prizeList.stream().filter(p->!prizeIdList.contains(p.getId())).collect(Collectors.toList());
            }
        }
        return prizeList;
    }



    @Override
    public List<DrawModel> listAllWinningInfo(PrizeDrawListRequest request) {
        Activity activity = activityService.getById(request.getActivityId());
        if(activity == null){
            throw  new BusinessException(ErrorEnum.SYSTEM_EXP, "未获取活动有效配置");
        }
        List<AccountPrize> prizeList = accountPrizeService.listAllWinningInfo(request.getSize(),request.getActivityId());
        return getDrawModels(prizeList);
    }

    /**
     * 统一封装
     *
     * @param prizeList
     * @return
     */
    private List<DrawModel> getDrawModels(List<AccountPrize> prizeList) {
        List<DrawModel> drawModelList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(prizeList)) {
            drawModelList = prizeList.stream().map(accountPrize -> {
                BeanCopier<DrawModel> copier = BeanCopier.create(accountPrize, DrawModel.builder().build(), CopyOptions.create());
                return copier.copy();
            }).collect(Collectors.toList());
        }

        return drawModelList;
    }

    /**
     * 抽奖
     *
     * @param prizeList
     * @return
     */
    private Prize selectCoupon(List<Prize> prizeList) {
        TreeMap<Double, Prize> weightMap = new TreeMap<>();
        for (Prize prize : prizeList) {
            //统一转为doubleni
            double lastWeight = weightMap.size() == 0 ? 0 : weightMap.lastKey();
            //权重累加
            weightMap.put(prize.getPrizeOdds().doubleValue() + lastWeight, prize);
        }
        double randomWeight = weightMap.lastKey() * Math.random();
        SortedMap<Double, Prize> tailMap = weightMap.tailMap(randomWeight, false);
        return weightMap.get(tailMap.firstKey());
    }



    /**
     *
     * 加锁可以
     *
     * @param phone
     * @return
     */
    private String buildKey(String phone) {
        StringBuffer sb = new StringBuffer();
        return sb.append(ConStants.SHIZU_DRAW_PREFIX).append(phone).toString().intern();
    }
}
