package com.miya.draw.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.miya.draw.entity.SyncTrade;
import com.miya.draw.entity.Trade;
import com.miya.draw.entity.TradeRbcs;
import com.miya.draw.mapper.TradeRbcsMapper;
import com.miya.draw.service.ITradeRbcsService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * @author ：djf
 * @date ：2022-12-09
 * @description：人本超市订单
 */
@Service
@DS("db2")
@Slf4j
@Transactional(propagation = Propagation.NOT_SUPPORTED)
public class ITradeRbcsServiceImpl extends ServiceImpl<TradeRbcsMapper, TradeRbcs> implements ITradeRbcsService{

    @Resource
    private TradeRbcsMapper tradeRbcsMapper;

    @Override
    public List<TradeRbcs> listByNeedSize(Long maxId, Integer maxSize) {
        LambdaQueryWrapper<TradeRbcs> wrapper = new LambdaQueryWrapper();
        wrapper.gt(TradeRbcs::getBillno, maxId);
//        wrapper.ge(Trade::getTrantime, LocalDateTimeUtil.beginOfDay(LocalDateTime.now()));
        wrapper.isNotNull(TradeRbcs::getIdentityid);
        wrapper.last("limit " + maxSize);
        return tradeRbcsMapper.selectList(wrapper);
    }

    @Override
    public boolean updateBatchByIdNoTrans(List<SyncTrade> syncTrades) {
        // 更新十足数据库出错了也就出错了
        List<TradeRbcs> batchTrade = new ArrayList<>();
        TradeRbcs trade;
        try {
            for (SyncTrade syncTrade : syncTrades) {
                trade = new TradeRbcs();
                trade.setBillno(Math.toIntExact(syncTrade.getBillNo()));
                trade.setStatus("Y");
                trade.setUpdatetime(LocalDateTime.now());
                batchTrade.add(trade);
            }
            return updateBatchById(batchTrade);
        } catch (Exception e) {
            log.error("回调更新十足数据库状态出错:{} list={}", e.getMessage(), JSON.toJSON(syncTrades));
        }
        return false;
    }

}
