package com.miya.draw.xxljob;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.google.common.collect.Lists;
import com.miya.draw.entity.SyncRecord;
import com.miya.draw.entity.SyncTrade;
import com.miya.draw.entity.Trade;
import com.miya.draw.entity.TradeRbcs;
import com.miya.draw.model.SyncTradeConfigModel;
import com.miya.draw.service.ISyncRecordService;
import com.miya.draw.service.ISyncTradeService;
import com.miya.draw.service.ITradeRbcsService;
import com.miya.draw.service.ITradeService;
import com.miya.draw.util.OrikaUtil;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.IJobHandler;
import com.xxl.job.core.handler.annotation.JobHandler;
import com.xxl.job.core.log.XxlJobLogger;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author ：djf
 * @date ：2022-12-08
 * @description：人本超市订单数据同步
 */
@Slf4j
@JobHandler("rbcsTradeSyncJob")
@Component
public class RbcsTradeSyncJob extends IJobHandler {
    static {
        OrikaUtil.classMap(TradeRbcs.class, SyncTrade.class)
                .field("billno", "billNo")
                .field("tradeidid", "tradeIdId")
                .field("storeid", "storeId")
                .field("storename", "storeName")
                .field("tradeno", "tradeNo")
                .field("posno", "posNo")
                .field("trantime", "tranTime")
                .field("stdamount", "stdAmount")
                .field("favamount", "favAmount")
                .field("memberid", "memberId")
                .field("identityid", "phone")
                .byDefault().register();
    }

    @Autowired
    private ITradeRbcsService tradeRbcsService;

    @Autowired
    private ISyncTradeService syncTradeService;

    @Autowired
    private ISyncRecordService syncRecordService;

    /**
     * 一次最多取多少数据
     */
    private Integer maxSize = 2000;

    /**
     * 每次存多少条数据
     */
    private Integer writeUnit = 500;

    /**
     * 商户Id
     */
    private Long mchId =14230L;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ReturnT<String> execute(String s) throws Exception {
        XxlJobLogger.log("十足同步开始:" + LocalDateTime.now().toString());
        if (StringUtils.isNotBlank(s)) {
            SyncTradeConfigModel model = JSON.parseObject(s, SyncTradeConfigModel.class);
            maxSize = model.getMaxSize();
            writeUnit = model.getWriteUnit();
            mchId = model.getMchId();
        }
        LambdaQueryWrapper<SyncRecord> wrapper = new LambdaQueryWrapper();
        wrapper.eq(SyncRecord::getMchId,mchId);
        wrapper.orderByDesc(SyncRecord::getId);
        wrapper.last("LIMIT 1");
        SyncRecord syncRecord = syncRecordService.getOne(wrapper);
        Long maxId = Objects.isNull(syncRecord) ? 0 : syncRecord.getLastSyncId();
        List<TradeRbcs> tradeList = tradeRbcsService.listByNeedSize(maxId, maxSize);
        List<SyncTrade> syncTradeList = OrikaUtil.convertList(tradeList, SyncTrade.class);
        // 升序
        syncTradeList = syncTradeList.stream().sorted(Comparator.comparing(SyncTrade::getBillNo))
                .collect(Collectors.toList());
        Long finalMchId = mchId;
        syncTradeList.forEach(trade->trade.setMchId(finalMchId));
        // 分割
        List<List<SyncTrade>> syncTradeParts = Lists.partition(syncTradeList, writeUnit);
        if (Objects.isNull(syncRecord)) {
            syncRecord = new SyncRecord();
        }

        for (List<SyncTrade> syncTrades : syncTradeParts) {
            // 批量更新
            syncTradeService.saveOrUpdateBatchByUniqKey(syncTrades);
            // 更新记录
            SyncTrade last = syncTrades.get(syncTrades.size() - 1);
            syncRecord.setMchId(mchId);
            syncRecord.setLastSyncId(last.getBillNo());
            syncRecord.setLastSyncTime(last.getTranTime());
            syncRecord.setUpdateTime(LocalDateTime.now());
            syncRecordService.saveOrUpdate(syncRecord);

            // 回写十足库
            tradeRbcsService.updateBatchByIdNoTrans(syncTrades);
        }

        XxlJobLogger.log("十足同步结束:" + LocalDateTime.now().toString());
        return ReturnT.SUCCESS;
    }
}
